@@ -4,7 +4,7 @@
 /* external variables */
 extern int softboot, watchdog, temp, maxtemp, tint, lastts, nrts;
 extern int maxload1, maxload5, maxload15, load, verbose, mem, minpages;
-extern int hbstamps;
+extern int hbstamps, logtick, ticker;
 extern pid_t pid;
 extern char *tempname, *admin, *devname, *progname, *timestamps, *heartbeat;
 extern time_t timeout;
@@ -25,7 +25,7 @@ struct filemode
 
 struct ifmode
 {
-	unsigned int bytes;
+	unsigned long bytes;
 };
 
 union wdog_options
