@@ -20,12 +20,14 @@
  *
  * Wed Oct  1 23:55:28 1997: Dick Streefland <dick_streefland@tasking.com>
  * Implemented the "bg", "fg" and "retry" mount options for NFS.
+ *
+ * 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ * - added Native Language Support
+ * 
+ * Modified by Olaf Kirch and Trond Myklebust for new NFS code,
+ * plus NFSv3 stuff.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 /*
  * nfsmount.c,v 1.1.1.1 1993/11/18 08:40:51 jrs Exp
  */
@@ -42,6 +44,7 @@
 #include <sys/time.h>
 #include <sys/utsname.h>
 #include <sys/stat.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 
 #include "sundries.h"
@@ -49,12 +52,23 @@
 
 #include <linux/nfs.h>
 #include "mount_constants.h"
-#include "nfs_mount3.h"
+#include "nfs_mount4.h"
+
+#define HAVE_inet_aton
+
+#ifndef NFS_PORT
+#define NFS_PORT 2049
+#endif
+#ifndef NFS_FHSIZE
+#define NFS_FHSIZE 32
+#endif
 
 static char *nfs_strerror(int stat);
 
 #define MAKE_VERSION(p,q,r)	(65536*(p) + 256*(q) + (r))
 
+#define MAX_NFSPROT ((nfs_mount_version >= 4) ? 3 : 2)
+
 static int
 linux_version_code(void) {
 	struct utsname my_utsname;
@@ -70,9 +84,9 @@ linux_version_code(void) {
 }
 
 /*
- * nfs_mount_version according to the kernel sources seen at compile time.
+ * nfs_mount_version according to the sources seen at compile time.
  */
-static int nfs_mount_version = KERNEL_NFS_MOUNT_VERSION;
+int nfs_mount_version = NFS_MOUNT_VERSION;
 
 /*
  * Unfortunately, the kernel prints annoying console messages
@@ -81,24 +95,82 @@ static int nfs_mount_version = KERNEL_NFS_MOUNT_VERSION;
  * and figure out what version the kernel expects.
  *
  * Variables:
- *	KERNEL_NFS_MOUNT_VERSION: kernel sources at compile time
  *	NFS_MOUNT_VERSION: these nfsmount sources at compile time
  *	nfs_mount_version: version this source and running kernel can handle
  */
 static void
 find_kernel_nfs_mount_version(void) {
-	int kernel_version = linux_version_code();
+	static int kernel_version = 0;
+
+	if (kernel_version)
+		return;
+
+	kernel_version = linux_version_code();
 
 	if (kernel_version) {
 	     if (kernel_version < MAKE_VERSION(2,1,32))
 		  nfs_mount_version = 1;
-	     else
+	     else if (kernel_version < MAKE_VERSION(2,2,18))
 		  nfs_mount_version = 3;
+	     else if (kernel_version < MAKE_VERSION(2,3,0))
+		  nfs_mount_version = 4; /* since 2.2.18pre9 */
+	     else if (kernel_version < MAKE_VERSION(2,3,99))
+		  nfs_mount_version = 3;
+	     else
+		  nfs_mount_version = 4; /* since 2.3.99pre4 */
 	}
 	if (nfs_mount_version > NFS_MOUNT_VERSION)
 	     nfs_mount_version = NFS_MOUNT_VERSION;
 }
 
+static struct pmap *
+get_mountport(struct sockaddr_in *server_addr,
+      long unsigned prog,
+      long unsigned version,
+      long unsigned proto,
+      long unsigned port)
+{
+	struct pmaplist *pmap;
+	static struct pmap p = {0, 0, 0, 0};
+
+	server_addr->sin_port = PMAPPORT;
+	pmap = pmap_getmaps(server_addr);
+
+	if (version > MAX_NFSPROT)
+		version = MAX_NFSPROT;
+	if (!prog)
+		prog = MOUNTPROG;
+	p.pm_prog = prog;
+	p.pm_vers = version;
+	p.pm_prot = proto;
+	p.pm_port = port;
+
+	while (pmap) {
+		if (pmap->pml_map.pm_prog != prog)
+			goto next;
+		if (!version && p.pm_vers > pmap->pml_map.pm_vers)
+			goto next;
+		if (version > 2 && pmap->pml_map.pm_vers != version)
+			goto next;
+		if (version && version <= 2 && pmap->pml_map.pm_vers > 2)
+			goto next;
+		if (pmap->pml_map.pm_vers > MAX_NFSPROT ||
+		    (proto && p.pm_prot && pmap->pml_map.pm_prot != proto) ||
+		    (port && pmap->pml_map.pm_port != port))
+			goto next;
+		memcpy(&p, &pmap->pml_map, sizeof(p));
+	next:
+		pmap = pmap->pml_next;
+	}
+	if (!p.pm_vers)
+		p.pm_vers = MOUNTVERS;
+	if (!p.pm_port)
+		p.pm_port = MOUNTPORT;
+	if (!p.pm_prot)
+		p.pm_prot = IPPROTO_TCP;
+	return &p;
+}
+
 int nfsmount(const char *spec, const char *node, int *flags,
 	     char **extra_opts, char **mount_opts, int running_bg)
 {
@@ -110,7 +182,6 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	char *old_opts;
 	char *mounthost=NULL;
 	char new_opts[1024];
-	fhandle root_fhandle;
 	struct timeval total_timeout;
 	enum clnt_stat clnt_stat;
 	static struct nfs_mount_data data;
@@ -119,13 +190,18 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	struct hostent *hp;
 	struct sockaddr_in server_addr;
 	struct sockaddr_in mount_server_addr;
+	struct pmap* pm_mnt;
 	int msock, fsock;
 	struct timeval retry_timeout;
-	struct fhstatus status;
+	union {
+		struct fhstatus nfsv2;
+		struct mountres3 nfsv3;
+	} status;
 	struct stat statbuf;
 	char *s;
 	int port;
 	int mountport;
+	int proto;
 	int bg;
 	int soft;
 	int intr;
@@ -133,6 +209,7 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	int nocto;
 	int noac;
 	int nolock;
+	int broken_suid;
 	int retry;
 	int tcp;
 	int mountprog;
@@ -151,22 +228,29 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	mclient = NULL;
 	if (strlen(spec) >= sizeof(hostdir)) {
 		fprintf(stderr, "mount: "
-			"excessively long host:dir argument\n");
+				  "excessively long host:dir argument\n");
 		goto fail;
 	}
 	strcpy(hostdir, spec);
-	if ((s = (strchr(hostdir, ':')))) {
+	if ((s = strchr(hostdir, ':'))) {
 		hostname = hostdir;
 		dirname = s + 1;
 		*s = '\0';
+		/* Ignore all but first hostname in replicated mounts
+		   until they can be fully supported. (mack@sgi.com) */
+		if ((s = strchr(hostdir, ','))) {
+			*s = '\0';
+			fprintf(stderr, "mount: warning: "
+					  "multiple hostnames not supported\n");
+		}
 	} else {
 		fprintf(stderr, "mount: "
-			"directory to mount not in host:dir format\n");
+				  "directory to mount not in host:dir format\n");
 		goto fail;
 	}
 
 	server_addr.sin_family = AF_INET;
-#if 1		/* old libc's do not have inet_aton() -- change 1 to 0 */
+#ifdef HAVE_inet_aton
 	if (!inet_aton(hostname, &server_addr.sin_addr))
 #endif
 	{
@@ -195,7 +279,7 @@ int nfsmount(const char *spec, const char *node, int *flags,
 		old_opts = "";
 	if (strlen(old_opts) + strlen(s) + 10 >= sizeof(new_opts)) {
 		fprintf(stderr, "mount: "
-			"excessively long option argument\n");
+				  "excessively long option argument\n");
 		goto fail;
 	}
 	sprintf(new_opts, "%s%saddr=%s",
@@ -222,16 +306,17 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	posix = 0;
 	nocto = 0;
 	nolock = 0;
+	broken_suid = 0;
 	noac = 0;
 	retry = 10000;		/* 10000 minutes ~ 1 week */
 	tcp = 0;
 
 	mountprog = MOUNTPROG;
-	mountvers = MOUNTVERS;
+	mountvers = 0;
 	port = 0;
 	mountport = 0;
 	nfsprog = NFS_PROGRAM;
-	nfsvers = NFS_VERSION;
+	nfsvers = 0;
 
 	/* parse options */
 
@@ -269,28 +354,35 @@ int nfsmount(const char *spec, const char *node, int *flags,
 			        mountport = val;
 			else if (!strcmp(opt, "mounthost"))
 			        mounthost=xstrndup(opteq+1,
-						  strcspn(opteq+1," \t\n\r,"));
+						   strcspn(opteq+1," \t\n\r,"));
 			else if (!strcmp(opt, "mountprog"))
 				mountprog = val;
 			else if (!strcmp(opt, "mountvers"))
 				mountvers = val;
 			else if (!strcmp(opt, "nfsprog"))
 				nfsprog = val;
-			else if (!strcmp(opt, "nfsvers"))
+			else if (!strcmp(opt, "nfsvers") ||
+				 !strcmp(opt, "vers"))
 				nfsvers = val;
-			else if (!strcmp(opt, "namlen")) {
+			else if (!strcmp(opt, "proto")) {
+				if (!strncmp(opteq+1, "tcp", 3))
+					tcp = 1;
+				else if (!strncmp(opteq+1, "udp", 3))
+					tcp = 0;
+				else
+					printf("Warning: Unrecognized proto= option.\n");
+			} else if (!strcmp(opt, "namlen")) {
 #if NFS_MOUNT_VERSION >= 2
 				if (nfs_mount_version >= 2)
 					data.namlen = val;
 				else
 #endif
-				printf("Warning: Option namlen is not supported.\n");
-			}
-			else if (!strcmp(opt, "addr"))
+					printf("Warning: Option namlen is not supported.\n");
+			} else if (!strcmp(opt, "addr"))
 				/* ignore */;
 			else {
 				printf("unknown nfs mount parameter: "
-				       "%s=%d\n", opt, val);
+					 "%s=%d\n", opt, val);
 				goto fail;
 			}
 		}
@@ -325,15 +417,19 @@ int nfsmount(const char *spec, const char *node, int *flags,
 					nolock = !val;
 				else
 					printf("Warning: option nolock is not supported.\n");
+			} else if (!strcmp(opt, "broken_suid")) {
+				broken_suid = val;
 			} else {
 				if (!sloppy) {
 					printf("unknown nfs mount option: "
-					       "%s%s\n", val ? "" : "no", opt);
+						 "%s%s\n", val ? "" : "no", opt);
 					goto fail;
 				}
 			}
 		}
 	}
+	proto = (tcp) ? IPPROTO_TCP : IPPROTO_UDP;
+
 	data.flags = (soft ? NFS_MOUNT_SOFT : 0)
 		| (intr ? NFS_MOUNT_INTR : 0)
 		| (posix ? NFS_MOUNT_POSIX : 0)
@@ -347,6 +443,23 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	if (nfs_mount_version >= 3)
 		data.flags |= (nolock ? NFS_MOUNT_NONLM : 0);
 #endif
+#if NFS_MOUNT_VERSION >= 4
+	if (nfs_mount_version >= 4)
+		data.flags |= (broken_suid ? NFS_MOUNT_BROKEN_SUID : 0);
+#endif
+	if (nfsvers > MAX_NFSPROT) {
+		fprintf(stderr, "NFSv%d not supported!\n", nfsvers);
+		return 0;
+	}
+	if (mountvers > MAX_NFSPROT) {
+		fprintf(stderr, "NFSv%d not supported!\n", nfsvers);
+		return 0;
+	}
+	if (nfsvers && !mountvers)
+		mountvers = (nfsvers < 3) ? 1 : nfsvers;
+	if (nfsvers && nfsvers < mountvers) {
+		mountvers = nfsvers;
+	}
 
 	/* Adjust options if none specified */
 	if (!data.timeo)
@@ -354,22 +467,22 @@ int nfsmount(const char *spec, const char *node, int *flags,
 
 #ifdef NFS_MOUNT_DEBUG
 	printf("rsize = %d, wsize = %d, timeo = %d, retrans = %d\n",
-		data.rsize, data.wsize, data.timeo, data.retrans);
+	       data.rsize, data.wsize, data.timeo, data.retrans);
 	printf("acreg (min, max) = (%d, %d), acdir (min, max) = (%d, %d)\n",
-		data.acregmin, data.acregmax, data.acdirmin, data.acdirmax);
+	       data.acregmin, data.acregmax, data.acdirmin, data.acdirmax);
 	printf("port = %d, bg = %d, retry = %d, flags = %.8x\n",
-		port, bg, retry, data.flags);
+	       port, bg, retry, data.flags);
 	printf("mountprog = %d, mountvers = %d, nfsprog = %d, nfsvers = %d\n",
-		mountprog, mountvers, nfsprog, nfsvers);
+	       mountprog, mountvers, nfsprog, nfsvers);
 	printf("soft = %d, intr = %d, posix = %d, nocto = %d, noac = %d\n",
-		(data.flags & NFS_MOUNT_SOFT) != 0,
-		(data.flags & NFS_MOUNT_INTR) != 0,
-		(data.flags & NFS_MOUNT_POSIX) != 0,
-		(data.flags & NFS_MOUNT_NOCTO) != 0,
-		(data.flags & NFS_MOUNT_NOAC) != 0);
+	       (data.flags & NFS_MOUNT_SOFT) != 0,
+	       (data.flags & NFS_MOUNT_INTR) != 0,
+	       (data.flags & NFS_MOUNT_POSIX) != 0,
+	       (data.flags & NFS_MOUNT_NOCTO) != 0,
+	       (data.flags & NFS_MOUNT_NOAC) != 0);
 #if NFS_MOUNT_VERSION >= 2
 	printf("tcp = %d\n",
-		(data.flags & NFS_MOUNT_TCP) != 0);
+	       (data.flags & NFS_MOUNT_TCP) != 0);
 #endif
 #endif
 
@@ -394,25 +507,25 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	/* create mount deamon client */
 	/* See if the nfs host = mount host. */
 	if (mounthost) {
-	  if (mounthost[0] >= '0' && mounthost[0] <= '9') {
-	    mount_server_addr.sin_family = AF_INET;
-	    mount_server_addr.sin_addr.s_addr = inet_addr(hostname);
-	  } else {
-		  if ((hp = gethostbyname(mounthost)) == NULL) {
-			  fprintf(stderr, "mount: can't get address for %s\n",
-				  hostname);
-			  goto fail;
-		  } else {
-			  if (hp->h_length > sizeof(struct in_addr)) {
-				  fprintf(stderr,
-					  "mount: got bad hp->h_length?\n");
-				  hp->h_length = sizeof(struct in_addr);
-			  }
-			  mount_server_addr.sin_family = AF_INET;
-			  memcpy(&mount_server_addr.sin_addr,
-				 hp->h_addr, hp->h_length);
-		  }
-	  }
+		if (mounthost[0] >= '0' && mounthost[0] <= '9') {
+			mount_server_addr.sin_family = AF_INET;
+			mount_server_addr.sin_addr.s_addr = inet_addr(hostname);
+		} else {
+			if ((hp = gethostbyname(mounthost)) == NULL) {
+				fprintf(stderr, "mount: can't get address for %s\n",
+					hostname);
+				goto fail;
+			} else {
+				if (hp->h_length > sizeof(struct in_addr)) {
+					fprintf(stderr,
+						"mount: got bad hp->h_length?\n");
+					hp->h_length = sizeof(struct in_addr);
+				}
+				mount_server_addr.sin_family = AF_INET;
+				memcpy(&mount_server_addr.sin_addr,
+				       hp->h_addr, hp->h_length);
+			}
+		}
 	}
 
 	/*
@@ -439,8 +552,10 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	prevt = 0;
 	t = 30;
 	val = 1;
+
 	for (;;) {
 		if (bg && stat(node, &statbuf) == -1) {
+			/* no mount point yet - sleep */
 			if (running_bg) {
 				sleep(val);	/* 1, 2, 4, 8, 16, 30, ... */
 				val *= 2;
@@ -452,34 +567,73 @@ int nfsmount(const char *spec, const char *node, int *flags,
 			if (t - prevt < 30)
 				sleep(30);
 
-			/* contact the mount daemon via TCP */
-			mount_server_addr.sin_port = htons(mountport);
-			msock = RPC_ANYSOCK;
-			mclient = clnttcp_create(&mount_server_addr,
-						 mountprog, mountvers,
-						 &msock, 0, 0);
+			pm_mnt = get_mountport(&mount_server_addr,
+					       mountprog,
+					       mountvers,
+					       proto,
+					       mountport);
 
-			/* if this fails, contact the mount daemon via UDP */
-			if (!mclient) {
-				mount_server_addr.sin_port = htons(mountport);
-				msock = RPC_ANYSOCK;
+			/* contact the mount daemon via TCP */
+			mount_server_addr.sin_port = htons(pm_mnt->pm_port);
+			msock = RPC_ANYSOCK;
+
+			switch (pm_mnt->pm_prot) {
+			case IPPROTO_UDP:
 				mclient = clntudp_create(&mount_server_addr,
-							 mountprog, mountvers,
-							 retry_timeout, &msock);
+							 pm_mnt->pm_prog,
+							 pm_mnt->pm_vers,
+							 retry_timeout,
+							 &msock);
+				if (mclient)
+					break;
+				mount_server_addr.sin_port = htons(pm_mnt->pm_port);
+				msock = RPC_ANYSOCK;
+			case IPPROTO_TCP:
+				mclient = clnttcp_create(&mount_server_addr,
+							 pm_mnt->pm_prog,
+							 pm_mnt->pm_vers,
+							 &msock, 0, 0);
+				break;
+			default:
+				mclient = 0;
 			}
+
 			if (mclient) {
 				/* try to mount hostname:dirname */
 				mclient->cl_auth = authunix_create_default();
-				clnt_stat = clnt_call(mclient, MOUNTPROC_MNT,
-					(xdrproc_t) xdr_dirpath, (caddr_t) &dirname,
-					(xdrproc_t) xdr_fhstatus, (caddr_t) &status,
-					total_timeout);
+
+				/* make pointers in xdr_mountres3 NULL so
+				 * that xdr_array allocates memory for us
+				 */
+				memset(&status, 0, sizeof(status));
+
+				if (pm_mnt->pm_vers == 3)
+					clnt_stat = clnt_call(mclient,
+						     MOUNTPROC3_MNT,
+						     (xdrproc_t) xdr_dirpath,
+						     (caddr_t) &dirname,
+						     (xdrproc_t) xdr_mountres3,
+						     (caddr_t) &status,
+						     total_timeout);
+				else
+					clnt_stat = clnt_call(mclient,
+						     MOUNTPROC_MNT,
+						     (xdrproc_t) xdr_dirpath,
+						     (caddr_t) &dirname,
+						     (xdrproc_t) xdr_fhstatus,
+						     (caddr_t) &status,
+						     total_timeout);
+
 				if (clnt_stat == RPC_SUCCESS)
 					break;		/* we're done */
+#if 0
+				/* errno? who sets errno? */
+				/* this fragment breaks bg mounting */
 				if (errno != ECONNREFUSED) {
 					clnt_perror(mclient, "mount");
 					goto fail;	/* don't retry */
 				}
+#endif
 				if (!running_bg && prevt == 0)
 					clnt_perror(mclient, "mount");
 				auth_destroy(mclient->cl_auth);
@@ -492,6 +646,7 @@ int nfsmount(const char *spec, const char *node, int *flags,
 			}
 			prevt = t;
 		}
+
 		if (!bg)
 		        goto fail;
 		if (!running_bg) {
@@ -504,15 +659,46 @@ int nfsmount(const char *spec, const char *node, int *flags,
 		if (t >= timeout)
 			goto fail;
 	}
+	nfsvers = (pm_mnt->pm_vers < 2) ? 2 : pm_mnt->pm_vers;
 
-	if (status.fhs_status != 0) {
-		fprintf(stderr,
-			"mount: %s:%s failed, reason given by server: %s\n",
-			hostname, dirname, nfs_strerror(status.fhs_status));
-		goto fail;
+	if (nfsvers == 2) {
+		if (status.nfsv2.fhs_status != 0) {
+			fprintf(stderr,
+				"mount: %s:%s failed, reason given by server: %s\n",
+				hostname, dirname,
+				nfs_strerror(status.nfsv2.fhs_status));
+			goto fail;
+		}
+		memcpy(data.root.data,
+		       (char *) status.nfsv2.fhstatus_u.fhs_fhandle,
+		       NFS_FHSIZE);
+#if NFS_MOUNT_VERSION >= 4
+		data.root.size = NFS_FHSIZE;
+		memcpy(data.old_root.data,
+		       (char *) status.nfsv2.fhstatus_u.fhs_fhandle,
+		       NFS_FHSIZE);
+#endif
+	} else {
+#if NFS_MOUNT_VERSION >= 4
+		fhandle3 *fhandle;
+		if (status.nfsv3.fhs_status != 0) {
+			fprintf(stderr,
+				"mount: %s:%s failed, reason given by server: %s\n",
+				hostname, dirname,
+				nfs_strerror(status.nfsv3.fhs_status));
+			goto fail;
+		}
+		fhandle = &status.nfsv3.mountres3_u.mountinfo.fhandle;
+		memset(data.old_root.data, 0, NFS_FHSIZE);
+		memset(&data.root, 0, sizeof(data.root));
+		data.root.size = fhandle->fhandle3_len;
+		memcpy(data.root.data,
+		       (char *) fhandle->fhandle3_val,
+		       fhandle->fhandle3_len);
+
+		data.flags |= NFS_MOUNT_VER3;
+#endif
 	}
-	memcpy((char *) &root_fhandle, (char *) status.fhstatus_u.fhs_fhandle,
-		sizeof (root_fhandle));
 
 	/* create nfs socket for kernel */
 
@@ -535,7 +721,7 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	if (port == 0) {
 		server_addr.sin_port = PMAPPORT;
 		port = pmap_getport(&server_addr, nfsprog, nfsvers,
-			tcp ? IPPROTO_TCP : IPPROTO_UDP);
+				    tcp ? IPPROTO_TCP : IPPROTO_UDP);
 		if (port == 0)
 			port = NFS_PORT;
 #ifdef NFS_MOUNT_DEBUG
@@ -547,11 +733,11 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	printf("using port %d for nfs deamon\n", port);
 #endif
 	server_addr.sin_port = htons(port);
-	 /*
-	  * connect() the socket for kernels 1.3.10 and below only,
-	  * to avoid problems with multihomed hosts.
-	  * --Swen
-	  */
+	/*
+	 * connect() the socket for kernels 1.3.10 and below only,
+	 * to avoid problems with multihomed hosts.
+	 * --Swen
+	 */
 	if (linux_version_code() <= 66314
 	    && connect(fsock, (struct sockaddr *) &server_addr,
 		       sizeof (server_addr)) < 0) {
@@ -562,8 +748,6 @@ int nfsmount(const char *spec, const char *node, int *flags,
 	/* prepare data structure for kernel */
 
 	data.fd = fsock;
-	memcpy((char *) &data.root, (char *) &root_fhandle,
-		sizeof (root_fhandle));
 	memcpy((char *) &data.addr, (char *) &server_addr, sizeof(data.addr));
 	strncpy(data.hostname, hostname, sizeof(data.hostname));
 
@@ -576,7 +760,7 @@ int nfsmount(const char *spec, const char *node, int *flags,
 
 	/* abort */
 
-fail:
+ fail:
 	if (msock != -1) {
 		if (mclient) {
 			auth_destroy(mclient->cl_auth);
