@@ -102,7 +102,7 @@ int check_pidfile(struct list *file)
     
 #if USE_SYSLOG
     /* do verbose logging */
-    if (verbose)
+    if (verbose && logtick && ticker == 1)
 	syslog(LOG_INFO, "was able to ping process %d (%s).", pid, file->name);
 #endif
 
