@@ -68,7 +68,7 @@ int verbose = FALSE;
 
 pid_t pid;
 int softboot = FALSE, watchdog = -1, load = -1, mem = -1, temp = -1;
-int tint = 10, logtick = 1, ticker = 0, schedprio = 1;
+int tint = 10, logtick = 1, ticker = 1, schedprio = 1;
 int maxload1 = 0, maxload5 = 0, maxload15 = 0, minpages = 0;
 int maxtemp = 120, hbstamps = 300, lastts, nrts;
 int pingcount = 3;
@@ -295,6 +295,11 @@ static void read_config(char *filename, char *progname)
 		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
 		else
 		    add_list(&pidfile, strdup(line + i));
+	    } else if (strncmp(line + i, PINGCOUNT, strlen(PINGCOUNT)) == 0) {
+		if (spool(line, &i, strlen(PINGCOUNT)))
+		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
+		else
+		    pingcount = atol(line + i);
 	    } else if (strncmp(line + i, PING, strlen(PING)) == 0) {
 		if (spool(line, &i, strlen(PING)))
 		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
@@ -398,11 +403,6 @@ static void read_config(char *filename, char *progname)
 		else {
 			minpages = atol(line + i);
 		}
-	    } else if (strncmp(line + i, PINGCOUNT, strlen(PINGCOUNT)) == 0) {
-		if (spool(line, &i, strlen(PINGCOUNT)))
-		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
-		else
-		    pingcount = atol(line + i);
 	    } else {
 		fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
 	    }
@@ -831,17 +831,13 @@ int main(int argc, char *const argv[])
 	/* finally sleep some seconds */
 	sleep(tint);
 
-	if(logtick && (--ticker == 0)) {
-	  ticker = logtick;
-	  
 #if USE_SYSLOG
 	/* do verbose logging */
-	if (verbose) {
-	    count += logtick;
-		ticker = 0;
-	    syslog(LOG_INFO, "still alive after %ld seconds = %ld interval(s)", count * tint, count);
+	if (verbose && logtick && (--ticker == 0)) {
+		ticker = logtick;
+	  	count += logtick;
+	    	syslog(LOG_INFO, "still alive after %ld interval(s)", count);
 	}
 #endif				/* USE_SYSLOG */
-	}
     }
 }
